# Copyright (c) 2015, InfluxFramework LLC
# License: MIT. See LICENSE

import json

import influxframework
from influxframework.model.document import Document
from influxframework.utils.jinja import validate_template


class EmailTemplate(Document):
	def validate(self):
		if self.use_html:
			validate_template(self.response_html)
		else:
			validate_template(self.response)

	def get_formatted_subject(self, doc):
		return influxframework.render_template(self.subject, doc)

	def get_formatted_response(self, doc):
		if self.use_html:
			return influxframework.render_template(self.response_html, doc)

		return influxframework.render_template(self.response, doc)

	def get_formatted_email(self, doc):
		if isinstance(doc, str):
			doc = json.loads(doc)

		return {"subject": self.get_formatted_subject(doc), "message": self.get_formatted_response(doc)}


@influxframework.whitelist()
def get_email_template(template_name, doc):
	"""Returns the processed HTML of a email template with the given doc"""
	if isinstance(doc, str):
		doc = json.loads(doc)

	email_template = influxframework.get_doc("Email Template", template_name)
	return email_template.get_formatted_email(doc)
